#!/usr/bin/env python

# $Id$

"""
jaraco.windows

A lightweight wrapper to provide a pythonic API to the Windows Platform.

This package attempts to provide interfaces similar or compatible
with Mark Hammond's pywin32 library, but avoids the use of extension
modules by utilizing ctypes.
"""
